@@ -116,7 +116,7 @@ void YClientContainer::handleButton(const XButtonEvent &button) {
     if (doRaise && (!doActivate || !raiseOnFocus))
         getFrame()->wmRaise();
     ///!!! it might be nice if this was per-window option (app-request)
-    if (!firstClick || passFirstClickToClient)
+    if (!firstClick || passFirstClickToClient || !getFrame()->client()->adopted())
         XAllowEvents(xapp->display(), ReplayPointer, CurrentTime);
     else
         XAllowEvents(xapp->display(), AsyncPointer, CurrentTime);
