@@ -187,9 +187,9 @@ void YMenu::focusItem(int itemNo) {
 
                 int ny = y();
                 if (ny + iy + ih > dy + dh)
-                    ny = dx + dh - ih - iy;
+                    ny = dy + dh - ih - iy;
                 else if (ny + iy < dy)
-                    ny = -iy;
+                    ny = dy - iy;
                 setPosition(x(), ny);
             }
         }
